# -*- coding: utf-8 -*-
import os
import datetime

def is_cached(file_path):
    '''
    Checks if the file cached is still valid
    '''
    if not os.path.exists(file_path):
        return False

    file_time = datetime.datetime.fromtimestamp(os.path.getctime(file_path))
    current_time = datetime.datetime.now()
    file_age = (current_time - file_time).total_seconds()

    if file_age > 86400*10:
        return False
    else:
        return True


# -*- coding: utf-8 -*-
import csv
import os
import re

import requests
from hyper.contrib import HTTP20Adapter

from spiders.common.objects import Symbol
from spiders.util import is_cached

headers = {
    "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8",
    "Accept-Encoding": "gzip, deflate",
    "Accept-Language": "en-GB,en;q=0.9,en-US;q=0.8,ml;q=0.7",
    "Connection": "keep-alive",
    "Host": "www.nasdaq.com",
    "Referer": "http://www.nasdaq.com",
    "Upgrade-Insecure-Requests": "1",
    "User-Agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/64.0.3282.119 Safari/537.36"
}


def get_exchange_url(exchange):
    return ("http://www.nasdaq.com/screening/companies-by-industry.aspx?"
            "exchange={}&render=download".format(exchange.name))


def fetch_file(exchange):
    url = get_exchange_url(exchange)
    session = requests.session()
    session.mount(url, HTTP20Adapter())
    file_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "../data/", exchange.name)
    out = open(file_path, 'w', newline='\n')
    csv_writer = csv.writer(out, delimiter=',')
    for i in range(50):
        print("start crawler page " + str(i+1))
        url = "https://www.nasdaq.com/api/v1/screener?page="+str(i+1)+"&pageSize=200"
        req = session.request("GET", url,
                              headers=headers, verify=False)
        json_obj = req.json()
        print(json_obj)
        rows = json_obj['data']
        if rows is None:
            continue

        for row in rows:
            if row is None or 'ticker' not in row:
                continue
            ticker = row['ticker']
            company = row['company']
            marketCapGroup = row['marketCapGroup']
            sector = row['sectorName']
            gicsSector = row['gicsSector']
            csv_writer.writerow([ticker, company, marketCapGroup, sector, gicsSector])


def save_file(file_path, symbol_data):
    with open(file_path, "w") as saved_file:
        saved_file.write(symbol_data)


def read_symbol_list(symbol_data):
    symbol_list = list()
    symbol_data = re.split("\r?\n", symbol_data)
    # symbol,company,sector,industry,headquaters
    symbol_data = list(csv.reader(symbol_data, delimiter=','))
    # We need to cut off the the last row because it is a null string
    for row in symbol_data:
        if len(row) < 2:
            continue
        symbol_data = Symbol()
        symbol_data.symbol = row[0]
        symbol_data.company = row[1]
        symbol_data.sector = row[2]
        symbol_data.industry = row[3]
        symbol_list.append(symbol_data)
    return symbol_list


def get_exchange_symbols(exchange):
    """
     read symbols from exchange

    """
    file_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "../data/", exchange.name)
    print(file_path)
    if not is_cached(file_path):
        fetch_file(exchange)
    symbol_data = None
    with open(file_path, "r") as cached_file:
        symbol_data = cached_file.read()
    return read_symbol_list(symbol_data)



# -*- coding: utf-8 -*-
import csv
import json

import requests

from spiders.common.objects import Kline
from spiders.spider import Spider


class NasdaqSpider(Spider):
    def __init__(self):
        self.base_url = 'https://api.nasdaq.com/api/quote/{0}/chart?assetclass=stocks&fromdate={1}&todate={2}'

    """
     get specify stock data by start and end date
     @:return kline
    """

    def get_stock_data(self, symbol, start, end):
        headers = {
            "authority": 'api.nasdaq.com',
            "Accept": "application/json, text/plain, */*",
            "Accept-Encoding": "deflate",
            "Accept-Language": "en-GB,en;q=0.9,en-US;q=0.8,ml;q=0.7",
            "Connection": "keep-alive",
            "Origin": "https://www.nasdaq.com",
            "sec-fetch-mode": "cors",
            "sec-fetch-site": "same-site",
            "Referer": "https://www.nasdaq.com/market-activity/stocks/amzn",
            "User-Agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/64.0.3282.119 Safari/537.36"
        }
        url = self.base_url.format(symbol, start, end)
        resp = requests.get(url, headers=headers, verify=False)
        klines = []
        if resp.json()['data'] is None or  resp.json()['data']['chart'] is None:
            print(symbol+" is none!")
            return klines

        stock_datas = resp.json()['data']['chart']
        if stock_datas and len(stock_datas) > 0:
            print(json.dumps(stock_datas))
            for stock_data in stock_datas:
                stock_info = stock_data['z']
                kline = Kline()
                kline.symbol = symbol
                kline.high = float(stock_info['high'])
                kline.low = float(stock_info['low'])
                kline.open = float(stock_info['open'])
                kline.close = float(stock_info['close'])
                kline.volume = float(stock_info['volume'].replace(',', ''))
                kline.value = float(stock_info['value'])
                kline.dateTime = stock_info['dateTime']
                klines.append(kline)
        return klines

    def store_kline_data(self, file_path, symbol, start, end):
        klines = self.get_stock_data(symbol=symbol, start=start, end=end)
        if len(klines) == 0:
            print("can't get klines")
            return
        with open(file_path, 'w', newline='') as csvfile:
            kline_writer = csv.writer(csvfile, delimiter=',',
                                      quotechar='|', quoting=csv.QUOTE_MINIMAL)
            kline_writer.writerow(['symbol', 'open', 'high', 'close', 'low', 'volume', 'date'])
            for kline in klines:
                kline_writer.writerow(
                    [kline.symbol, kline.open, kline.high, kline.close, kline.low, kline.volume, kline.dateTime])



from distutils.core import setup
import setuptools

setup(
    name='stock-spider',
    version='1.0.4',
    packages=['spiders'],
    author='xbkaishui',
    author_email='xbkaishui@126.com',
    description='us stock spider',
    # long_description=open('README.md').read(),
    # long_description_content_type="text/markdown",
    url='http://xbkaishui.github.io/',
    download_url='http://pypi.python.org/pypi/stock-spider',
    keywords='Spider US Stock'.split(),
    license='Apache License 2.0',
    install_requires=[
        "requests >= 2.20.0",
        "hyper >= 0.7.0",
        "dataclasses >= 0.6"
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Lesser General Public License v2 or later (LGPLv2+)',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Office/Business :: Financial :: Investment',
    ]
)



# -*- coding: utf-8 -*-

from spiders.symbols import get_exchange_symbols
from spiders.common.constant import Exchange
import os
from spiders.nasdaq import NasdaqSpider
from spiders.util import is_cached
from multiprocessing import Pool

class CrawlerEngine(object):

    def __init__(self):
        self.spider = NasdaqSpider()

    """
     sync all stock to local path, support multi thread
    """
    def sync_all_nasdaq(self, file_path, start, end, cache = True):
        symbols = get_exchange_symbols(Exchange.NASDAQ)
        print("symbols size " + str(len(symbols)))
        self.create_dir_if_needed(file_path)
        cpu_count = os.cpu_count()
        pool = Pool(cpu_count)
        sync_params = []
        for symbol in symbols:
            param = {"symbol":symbol, "file_path":file_path, "start":start, "end":end, "cache":cache}
            sync_params.append(param)
        pool.map(self.sync_single_symbol, sync_params)

    def create_dir_if_needed(self, file_path):
        if not os.path.exists(file_path):
            os.mkdir(file_path)

    def sync_single_symbol(self, param):
        symbol = param['symbol']
        file_path = param['file_path']
        start  = param['start']
        end = param['end']
        cache = param['cache']
        try:
            code = symbol.symbol
            stock_path = os.path.join(file_path, code+".csv")
            if cache and is_cached(stock_path):
                print("file is cached ignore " + stock_path)
                return
            print("try to sync symbol " + code)
            self.spider.store_kline_data(stock_path, code, start, end)
        except :
            print("fetch symbol error " + symbol.symbol + " ignore")



